use bevy::{picking::{focus::HoverMap, pointer::PointerId}, prelude::*, utils::HashMap, window::PrimaryWindow};

use crate::ui;

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup)
            .add_systems(Update, move_cam)
            .init_resource::<Pointers>();
    }
}

fn setup(mut cmds: Commands, window: Query<&Window>) {
    let zoom = 2./window.single().width().min(window.single().height());
    cmds.spawn((
        Camera2d,
        Transform::from_scale(Vec3::new(zoom, zoom, 1.))
    ));
}

#[derive(Resource, Default)]
struct Pointers(HashMap<PointerId, Vec2>);

fn move_cam(
    mut cam: Query<&mut Transform, With<Camera2d>>,
    map_ui_entity: Query<Entity, With<ui::MapUIComponent>>,
    mouse_buttons: Res<ButtonInput<MouseButton>>,
    touches: Res<Touches>,
    window: Query<&Window, With<PrimaryWindow>>,
    mut pointers: ResMut<Pointers>,
    hover_map: Res<HoverMap>
) {
    let window = window.single();
    let mut cam = cam.single_mut();
    let map_ui_entity = map_ui_entity.single();
    for (id, hit_map) in hover_map.iter() {
        if let Some((pressed, new_pos, old_pos)) = match id {
            PointerId::Mouse => window.cursor_position().map(|p| (mouse_buttons.pressed(MouseButton::Left), p, None)),
            PointerId::Touch(i) => touches.get_pressed(*i).map(|t| (true, t.position(), Some(t.previous_position()))),
            _ => None
        } {
            let old_pos = old_pos.unwrap_or(pointers.0.get(id).map(|p| *p).unwrap_or_default());
            // dbg!(pressed, &new_pos, &old_pos);
            if pressed {
                // dbg!(hit_map);
                for entity in hit_map.keys() {
                    dbg!(entity);
                    if *entity == map_ui_entity {
                        let delta = new_pos - old_pos;
                        cam.translation.x -= delta.x*cam.scale.x;
                        cam.translation.y += delta.y*cam.scale.y;
                    }
                }
            }
            pointers.0.insert(*id, new_pos);
        }
    }
}